@@ -39,9 +39,25 @@
 #include <sys/resource.h>
 #endif
 
+#ifdef	HAVE_SETLOCALE
+#include <locale.h>
+#else
+#define	setlocale(s, t)
+#endif
+
+#ifdef	HAVE_GETTEXT
+#include <libintl.h>
+#else
+#define dgettext(d, s) s
+#define	gettext(s)	s
+#define	bindtextdomain(s, t)
+#define	textdomain(s)
+#endif
+
 using namespace UCOMMON_NAMESPACE;
 
 OrderedIndex shell::Option::index;
+const char *shell::domain = NULL;
 
 shell::pipeio::pipeio()
 {
@@ -335,7 +351,7 @@ void shell::iobuf::open(const char *path, char **argv, shell::pmode_t mode, size
 	if(!pipeio::spawn(path, argv, mode, size, env))
 		IOBuffer::allocate(size, (type_t)mode);
 }
-		
+	
 shell::Option::Option(char shortopt, const char *longopt, const char *value, const char *help) :
 OrderedObject(&index)
 {
@@ -390,6 +406,7 @@ const char *shell::numericopt::assign(const char *value)
 	if(used)
 		return errmsg(shell::OPTION_USED);
 
+	used = true;
 	number = strtol(value, &endptr, 0);
 	if(!endptr || *endptr != 0)
 		return errmsg(shell::BAD_VALUE);
@@ -410,6 +427,7 @@ const char *shell::stringopt::assign(const char *value)
 		return shell::errmsg(shell::OPTION_USED);
 
 	text = value;
+	used = true;
 	return NULL;
 }
 
@@ -439,6 +457,7 @@ const char *shell::charopt::assign(const char *value)
 	if(used)
 		return shell::errmsg(shell::OPTION_USED);
 
+	used = true;
 	if(value[1] == 0) {
 		code = value[0];
 		return NULL;
@@ -493,6 +512,9 @@ void shell::set0(char *argv0)
 	if(ieq(ext, ".exe") || ieq(ext, ".com"))
 		*ext = 0;
 #endif
+
+	if(!domain)
+		bind(_argv0);
 }
 
 shell::shell(size_t pagesize) :
@@ -524,22 +546,22 @@ mempager(pagesize)
 }
 
 static const char *msgs[] = {
-	"missing command line arguments",
-	"missing argument for option",
-	"option does not have argument",
-	"unknown command option",
-	"option already used",
-	"invalid argument used",
+	_TEXT("missing command line arguments"),
+	_TEXT("missing argument for option"),
+	_TEXT("option does not have argument"),
+	_TEXT("unknown command option"),
+	_TEXT("option already used"),
+	_TEXT("invalid argument used"),
 	NULL};
 
 const char *shell::errmsg(errmsg_t id)
 {
-	return msgs[id];
+	return dgettext("ucommon", msgs[id]);
 }
 
 void shell::errmsg(errmsg_t id, const char *text)
 {
-	msgs[id] = text;
+	msgs[id] = shell::text(text);
 }
 
 unsigned shell::count(char **argv)
@@ -561,20 +583,30 @@ void shell::help(void)
 			op.next();
 			continue;
 		}
-		if(op->short_option && op->long_option)
+		if(op->short_option && op->long_option) {
 			printf("  -%c, ", op->short_option);
-		else if(op->long_option)
-			printf("     ");
-		else
-			printf("  -%c ", op->short_option);
-		hp = 5;
-		if(op->long_option && op->uses_option) {
-			printf("%s=%s", op->long_option, op->uses_option);
-			hp += strlen(op->long_option) + strlen(op->uses_option) + 1;
+			hp = 6;
 		}
 		else if(op->long_option) {
-			printf("%s", op->long_option);
-			hp += strlen(op->long_option);
+			printf("  ");
+			hp = 2;
+		}
+		else if(op->uses_option) {
+			printf("  -%c %s", op->short_option, op->uses_option);
+			hp = 5 + strlen(op->uses_option);
+		}
+		else {
+			printf("  -%c ", op->short_option);
+			hp = 5;
+		}
+	
+		if(op->long_option && op->uses_option) {
+			printf("--%s=%s", op->long_option, op->uses_option);
+			hp += strlen(op->long_option) + strlen(op->uses_option) + 3;
+		}
+		else if(op->long_option) {
+			printf("--%s", op->long_option);
+			hp += strlen(op->long_option) + 2;
 		}
 		if(hp > 29) {
 			printf("\n");
@@ -584,7 +616,7 @@ void shell::help(void)
 			putchar(' ');
 			++hp;
 		} 
-		const char *hs = op->help_string;
+		const char *hs = shell::text(op->help_string);
 		while(*hs) {
 			if(*hs == '\n' || (((*hs == ' ' || *hs == '\t')) && (hp > 75))) {
 				printf("\n                              ");
@@ -745,7 +777,7 @@ char **shell::getargv(char **argv)
 				errexit(1, "*** %s: --%s: %s\n", _argv0, op->long_option, errmsg(shell::NOARGUMENT));
 			err = op->assign(value);
 			if(err)
-				errexit(1, "*** %s: --%s: %s\n", _argv0, op->long_option, err);
+				errexit(1, "*** %s: --%s: %s\n", _argv0, op->long_option, shell::text(err));
 			continue;
 		}
 
@@ -771,22 +803,21 @@ char **shell::getargv(char **argv)
 			
 			if(!is(op))
 				errexit(1, "*** %s: -%c: %s\n", _argv0, *arg, errmsg(shell::BADOPTION));
-			if(op->uses_option && arg[1] == 0) {
+
+			value = NULL;
+
+			if(op->uses_option && arg[1] == 0)
 				value = argv[argp++];
-				break;
-			}
-			if(op->uses_option) {
+			else if(op->uses_option)
 				value = ++arg;
+
+			if(op->uses_option && value == NULL)
+				errexit(1, "*** %s: -%c: %s\n", _argv0, op->short_option, errmsg(shell::NOARGUMENT));
+			err = op->assign(value);
+			if(err)
+				errexit(1, "*** %s: -%c: %s\n", _argv0, op->short_option, shell::text(err));
+			if(op->uses_option)
 				break;
-			}
-	
-			if(is(op)) {
-				if(op->uses_option && value == NULL)
-					errexit(1, "*** %s: -%c: %s\n", _argv0, op->short_option, errmsg(shell::NOARGUMENT));
-				err = op->assign(value);
-				if(err)
-					errexit(1, "*** %s: -%c: %s\n", _argv0, op->short_option, err);
-			}
 		}
 	}
 	_argv = &argv[argp];
@@ -1530,4 +1561,44 @@ int shell::cancel(shell::pid_t pid)
 
 #endif
 
+const char *shell::texts(const char *singular, const char *plural, unsigned long value)
+{
+#ifdef	HAVE_GETTEXT
+	return ::ngettext(singular, plural, value);
+#else
+	if(value > 1)
+		return plural;
+
+	return singular;
+#endif
+}
+
+const char *shell::text(const char *msg)
+{
+#ifdef	HAVE_GETTEXT
+	return ::gettext(msg);
+#else
+	return msg;
+#endif
+}
+
+void shell::bind(const char *name)
+{
+	if(!domain) {
+		setlocale(LC_ALL, "");
+		bindtextdomain("ucommon", UCOMMON_LOCALE);
+	}
+
+	bindtextdomain(name, UCOMMON_LOCALE);
+	textdomain(name);
+	domain = name;
+}
+
+void shell::rebind(const char *name)
+{
+	if(name)
+		textdomain(name);
+	else
+		textdomain(domain);
+}
 
