@@ -829,6 +829,27 @@ public:
 	 */
 	bool operator>=(const char *text) const;
 
+	/**
+	 * Parse long integer value from a string.
+	 * @param value to store.
+	 * @return object in expression.
+	 */
+	string &operator%(long& value);
+
+	/**
+	 * Parse double value from a string.
+	 * @param value to store.
+	 * @return object in expression.
+	 */
+	string &operator%(double& value);
+
+	/**
+	 * Parse text from a string in a scan expression.
+	 * @param text to scan and bypass.
+	 * @return object in expression.
+	 */
+	string &operator%(const char *text);
+
 	/**
 	 * Scan input items from a string object.
 	 * @param object to scan from.
@@ -837,6 +858,7 @@ public:
 	 */
 	static int scanf(string& object, const char *format, ...) __SCANF(2, 3);
 
+
 	/**
 	 * Print formatted items into a string object.
 	 * @param object to print into.
@@ -1531,7 +1553,6 @@ protected:
 	virtual strsize_t getStringSize(void) const = 0;
 };
 
-
 /**
  * A template to create a character array that can be manipulated as a string.
  * This is a mini string/stringbuf class that supports a subset of
