@@ -77,28 +77,20 @@ static void codecs(void)
 
 static void usage(void)
 {
-	printf("Usage: audiotool [options] command files...\n"
-		"Options:\n"
-		"  -buffer value         Set buffering in millisec (120 default)\n"
-		"  -codecs               List names of supported encodings\n"
-		"  -encoding codec       Specify channel encoding, default is pcmu\n"
-		"  -framing value        Set framing in millisec (20 default)\n"
-		"  -help                 Display this information\n"
-		"  -note \"text\"          Text annotation for new files\n"
-		"  -output codec         Specify format for new files, default is pcmu\n"
-		"  -verbose              Display extra debugging and stats info\n"
-		"  -version              Display release version info\n"
-		"Commands:\n"
-		"  append file files     Append audio to existing file\n"
-		"  create newfile files  Create new file from existing ones\n"
-		"  info files            Display info for specified audio files\n"
+	printf("%s\n", _TEXT("Usage: audiotool [options] command files...\n"
+		"Options:\n"));
+	shell::help();
+	printf("%s", _TEXT("Commands:\n"
+		"  append file files           Append audio to existing file\n"
+		"  create newfile files        Create new file from existing ones\n"
+		"  info files                  Display info for specified audio files\n"
 #ifndef	_MSWINDOWS_
-		"  pipe files            Pipe raw transcoded audio to stdout\n"
+		"  pipe files                  Pipe raw transcoded audio to stdout\n"
 #endif
-		"  text file             Display annotation for audio file if set\n"
-		"  verify file           Verify readability of an audio file\n"
-	);
-	printf("Report bugs to sipwitch-devel@gnu.org\n");
+		"  text file                   Display annotation for audio file if set\n"
+		"  verify file                 Verify readability of an audio file\n"
+	));
+	printf("%s\n", _TEXT("Report bugs to sipwitch-devel@gnu.org"));
     exit(0);
 }
 	
@@ -381,72 +373,62 @@ static void verify(const char *fn)
 	}
 }
 
-static long getvalue(const char *cp, const char *opt)
-{
-	if(!cp || !*cp) {
-		fprintf(stderr, "*** %s: missing value\n", opt);
-		exit(3);
-	}
-
-	return atol(cp);
-}
-
 int main(int argc, char **argv)
 {
-    char *cp;
+    char *cp = NULL;
 
-    if(argc < 2) {
-        fprintf(stderr, "use: audiotool [options] command files...\n");
-        exit(4);
-    }
+	shell::bind("audiotool");
+
+	shell::flagopt helpflag('h', "--help", _TEXT("display this list"));
+	shell::flagopt althelp('?', NULL, NULL);
+		shell::numericopt bufopt('b', "--buffering", _TEXT("audio buffering (120)"), "msec", 120);
+	shell::flagopt codecflag('c', "--codecs", _TEXT("list codecs"));
+	shell::stringopt encopt('e', "--encoding", _TEXT("channel encoding (pcmu)"), "codec", "pcmu");
+	shell::numericopt frameopt('f', "--framing", _TEXT("audio framing (20)"), "msec", 20); 
+	shell::stringopt noteopt('n', "--note", _TEXT("text annotation for new files"), "\"text\"", "");
+	shell::stringopt outopt('o', "--output", _TEXT("encoding for new files (pcmu)"), "codec", "pcmu");
+	shell::flagopt vflag('v', "--verbose", _TEXT("display extra debugging info"));
+	shell::flagopt verflag(0, "--version", _TEXT("display software version"));
+	shell args(argc, argv);
 
 	audio::init();
 
-	// first check for options....
+	if(is(helpflag) || is(althelp))
+		usage();
 
-	while(NULL != (cp = *++argv)) {
-		if(!strncmp(cp, "--", 2))
-			++cp;
+	if(is(verflag))
+		version();
 
-		if(*cp != '-')
-			break;
+	if(is(codecflag))
+		codecs();
 
-		if(eq(cp, "-version"))
-			version();
-		else if(eq(cp, "-help"))
-			usage();
-		else if(eq(cp, "-codecs"))
-			codecs();
-		else if(eq(cp, "-buffer"))
-			buffering = getvalue(*(++argv), "-buffer");
-		else if(eq(cp, "-verbose"))
-			verbose = true;
-		else if(eq(cp, "-format") || eq(cp, "-encoding")) {
-			encoding = audiocodec::find(*(++argv));
-			if(!encoding) {
-				fprintf(stderr, "*** %s: unknown encoding\n", *argv);
-				codecs();
-				exit(4);
-			}
-		}
-		else if(eq(cp, "-oformat") || eq(cp, "-output")) {
-			oformat = audiocodec::find(*(++argv));
-			if(!oformat) {
-				fprintf(stderr, "*** %s: unknown encoding\n", *argv);
-				codecs();
-				exit(4);
-			}
-		}
-		else if(eq(cp, "-note"))
-			note = *(++argv);
-		else if(eq(cp, "-framing"))
-			framing = getvalue(*(++argv), "-framing");
-		else {
-			fprintf(stderr, "*** %s: unknown option\n", cp);
-			exit(4);
-		}
+	if(is(bufopt)) {
+		buffering = (timeout_t) *bufopt;
+		if(buffering < 60 || buffering % 10)
+			shell::errexit(1, "*** audiotool: %s: %s\n", "--buffering", _TEXT("must be >= 60 msec and multiple of 10msec"));
 	}
 
+	if(is(frameopt)) {
+		framing = (timeout_t) *frameopt;
+		if(framing % 10 || framing < 10)
+			shell::errexit(1, "*** audiotool: %s: %s\n", "--framing",  _TEXT("must be multiple of 10 msec"));
+	}
+
+	if(is(encopt)) {
+		encoding = audiocodec::find(*encopt);
+		if(!encoding)
+			shell::errexit(1, _TEXT("*** audiotool: --encoding: %s: unknown encoding\n"), *encopt);
+	}
+
+	if(is(outopt)) {
+		oformat = audiocodec::find(*outopt);
+		if(!oformat)
+			shell::errexit(1, _TEXT("*** audiotool: --output: %s: unknown encoding\n"), *outopt);
+	}
+
+	if(is(noteopt))
+		note = *noteopt;
+
 	atexit(&stop);
 	if(!encoding)
 		encoding = audiocodec::find("pcmu");
@@ -463,6 +445,13 @@ int main(int argc, char **argv)
     signal(SIGABRT, down);
 #endif	
 
+	argv = args.argv();
+
+	if(argv && *argv)
+		cp = *argv;
+	else
+		usage();
+
 	// check commands...
 
 	while(*cp == '-')
@@ -522,15 +511,15 @@ int main(int argc, char **argv)
     }
 #endif
 
-    fprintf(stderr, "*** audiotool: %s: unknown command\n", cp);
+    fprintf(stderr, _TEXT("*** audiotool: %s: unknown command\n"), cp);
     exit(4);
 
 nofiles:
-	fprintf(stderr, "*** audiotool: %s: no file(s) to process\n", cp);
+	fprintf(stderr, _TEXT("*** audiotool: %s: no file(s) to process\n"), cp);
 	exit(4);
 
 toomany:
-	fprintf(stderr, "*** audiotool: %s: too many filenames used\n", cp);
+	fprintf(stderr, _TEXT("*** audiotool: %s: too many filenames used\n"), cp);
 	exit(4);
 
 done:
